package com.zipcodeconway;

import java.util.Random;

public class ConwayGameOfLife {

    int[][] currentMatrix;
    int[][] nextMatrix;
    SimpleWindow displayWindow;

    public ConwayGameOfLife(Integer dimension) {
        this.displayWindow = new SimpleWindow(dimension);
        currentMatrix = createRandomStart(dimension);
        nextMatrix = new int[dimension][dimension];
     }

    public ConwayGameOfLife(Integer dimension, int[][] startmatrix) {
        this.displayWindow = new SimpleWindow(dimension);
        currentMatrix = startmatrix;
        nextMatrix = new int[dimension][dimension];
    }

    public static void main(String[] args) {
        ConwayGameOfLife sim = new ConwayGameOfLife(50);
        int[][] endingWorld = sim.simulate(50);
    }

    // Contains the logic for the starting scenario.
    // Which cells are alive or dead in generation 0.
    // allocates and returns the starting matrix of size 'dimension'
    private int[][] createRandomStart(Integer dimension) {
        Random random = new Random();
        int[][] randomArray = new int[dimension][dimension];
        for (int i = 0; i < dimension; i++) {
            for(int j = 0; j < dimension; j++) {
                randomArray[i][j] = random.nextInt(2);
            }
        }
        return randomArray;
    }

    public int[][] simulate(Integer maxGenerations) {
        for (int generation = 0; generation <= maxGenerations; generation++) {
            this.displayWindow.display(currentMatrix, generation);
            for (int i = 0; i < currentMatrix.length; i++) {
                for (int j = 0; j < currentMatrix.length; j++) {
                    nextMatrix[i][j] = isAlive(i, j, currentMatrix);
                }
            }
            copyAndZeroOut(nextMatrix, currentMatrix);
            this.displayWindow.sleep(125);
        }
        return currentMatrix;
    }

    // copy the values of 'next' matrix to 'current' matrix,
    // and then zero out the contents of 'next' matrix
    public void copyAndZeroOut(int[][] next, int[][] current) {
        for (int i = 0; i < current.length; i++) {
            for (int j = 0; j < current.length; j++) {
                current[i][j] = next[i][j];
                next[i][j] = 0;
            }
        }
    }

    // Calculate if an individual cell should be alive in the next generation.
    // Based on the game logic:
	/*
		Any live cell with fewer than two live neighbours dies, as if by needs caused by underpopulation.
		Any live cell with more than three live neighbours dies, as if by overcrowding.
		Any live cell with two or three live neighbours lives, unchanged, to the next generation.
		Any dead cell with exactly three live neighbours cells will come to life.
	*/
    private int isAlive(int row, int col, int[][] world) {

        int neighborCount = countNeighbors(row, col, world);
        int aliveOrDead = 0;
        if (world [row][col] == 1) {
            if (neighborCount < 2 || neighborCount > 3) {
                aliveOrDead = 0;
            }
            else  {
                aliveOrDead = 1;
            }
        }
        else {
            if (neighborCount == 3) {
                aliveOrDead = 1;
            }
        }
        return aliveOrDead;
    }

    private int countNeighbors(int row, int col, int[][] world) {
        int count = 0;
        for (int i = row - 1; i < row + 2; i++) {
            for (int j = col - 1; j < col + 2; j++) {
                if ( (!(i == row && j == col)) && i >= 0 && j >= 0 && i < world.length && j < world.length) {
                    if (world[i][j] == 1) {
                        count++;
                    }
                }
            }
        }
        return count;
    }
}
